#! /bin/sh

config() {
  NEW="$1"
  OLD="`dirname $NEW`/`basename $NEW .new`"
  # If there's no config file by that name, mv it over:
  if [ ! -r $OLD ]; then
    mv $NEW $OLD
  elif [ "`cat $OLD | md5sum`" = "`cat $NEW | md5sum`" ]; then
    # toss the redundant copy
    rm $NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}

config etc/rc.d/rc.squid.new

for file in etc/squid/* ;  do
  config $file
done
config etc/logrotate.d/squid.new


# add a squid user and group
# Define minimum values for UID and GID
MUID=43
MGID=43
USERNAME=squid
GROUPNAME=squid
COMMENT=Squid
HOMEDIR=/var/spool/squid

(
if ! grep -qe "^$GROUPNAME:" etc/group; then
        NEXTGID="$(MGID=$MGID awk -F: '{uid[$3]=1} END { for (i=ENVIRON["MGID"];i in uid;i++);print i}' etc/group)"
        echo creating group $GROUPNAME with GID $NEXTGID...
        chroot . /usr/sbin/groupadd -g $NEXTGID $GROUPNAME
fi
)
(
if ! grep -qe "^$USERNAME:" etc/passwd; then
        NEXTUID="$(MUID=$MUID awk -F: '{uid[$3]=1} END { for (i=ENVIRON["MUID"];i in uid;i++);print i}' etc/passwd)"
        echo creating user $USERNAME with UID $NEXTUID...
        chroot . /usr/sbin/useradd -g $GROUPNAME -u $NEXTUID -d $HOMEDIR -c $COMMENT -s /bin/false $USERNAME
fi
)

# fix rights and ownership on the squid cache directory
chown -R $USERNAME:$GROUPNAME var/spool/squid
chmod 0750 var/spool/squid

chown -R $USERNAME:$GROUPNAME var/log/squid
chmod 0750 var/log/squid

( cd etc/squid ; rm -rf errors )
( cd etc/squid ; ln -sf /usr/share/squid/errors/English errors )
