#!/bin/bash
# Author : Guth (guth@ploposor.com)
# URL : http://www.squid-cache.org/Versions/v3/3.0/squid-3.0.STABLE13.tar.bz2


# program name/version
FilenameFromUrl="$(grep 'U[R]L' $0 | rev | cut -d '/' -f1 | rev)"
softname="$(echo $FilenameFromUrl | cut -d '-' -f1)"

# program name/version
softversion='3.0.13'
#softversion="$(echo $FilenameFromUrl | cut -d '-' -f2 | cut -d '.' -f -3)"
#softname='squid'

# architecture/package version
arch='i486'
cpu='i686'
packageversion='60.1'

# number of parallel tasks of make
ntasks='-j4'

# parameters needed for the build process
buildir=$(pwd)
package="$softname-$softversion-$arch-$packageversion"
srcpkg="$buildir/$FilenameFromUrl"
srcdir=`echo "$FilenameFromUrl" |rev | cut -d '.' -f 3- |cut -d '/' -f 1 | rev`

cd $buildir
rm -rf $srcdir

#if [ ! -x $srcdir.tar.bz2 ]; then
#    wget -c $srcurl;
#fi


if ! [ -f "$FilenameFromUrl" ]; then
    echo "Downloading the program tarball...$(grep 'U[R]L' $0 | rev | cut -d ' ' -f1 | cut -d "	" -f 1 | rev)"
    wget "$(grep 'U[R]L' $0 | rev | cut -d ' ' -f1 | cut -d "	" -f 1 | rev)" ||exit 0
fi




# prepare the build result directory
dest="$buildir/$package"
rm -rf $dest
rm -f $package.*
mkdir -p $dest
mkdir -p $dest/install
mkdir -p $dest/usr/src/$softname-$softversion

# make a slack-desc file
cat <<EOF > $dest/install/slack-desc
       |-----handy-ruler------------------------------------------------------|
squid: Squid (Web Proxy Cache)
squid:
squid: Squid is a high-performance proxy caching server for Web clients,
squid: supporting FTP, gopher, and HTTP data objects. Unlike traditional
squid: caching software, Squid handles all requests in a single,
squid: non-blocking, I/O-driven process. Squid keeps meta data
squid: and especially hot objects cached in RAM, caches DNS lookups,
squid: supports non-blocking DNS lookups, and implements
squid: negative caching of failed requests.
squid: 
squid: 
EOF

# extract sources
echo "extracting ..."

#srcurl="$(head $0 | grep -i location | cut -d ':' -f 2-)";

tar jxf $FilenameFromUrl || exit -1
cd $srcdir

# set correct permissions
chown -R root.root .
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;

# configure package for compilation

echo "configuring ..."

mkdir -p $dest/usr/doc/$softname-$softversion

export CFLAGS="-O2 -march=$arch -mtune=$cpu"
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"
./configure \
    --prefix=/usr \
    --sysconfdir=/etc/squid \
    --localstatedir=/var \
    --mandir=/usr/man \
    --with-large-files \
    --with-pthreads \
    --enable-icmp \
    --enable-delay-pools \
    --enable-icap-client \
    --enable-cachemgr-hostname=localhost \
    --enable-arp-acl \
    --enable-ssl \
    --enable-cache-digests \
    --enable-linux-netfilter \
    --enable-linux-tproxy \
    --enable-auth="basic,digest,ntlm,negotiate" \
    --enable-basic-auth-helpers="DB,LDAP,MSNT,NCSA,POP3,SASL,SMB,mswin_sspi,multi-domain-NTLM,squid_radius_auth" \
    --enable-ntlm-auth-helpers="SMB,fakeauth,mswin_sspi,no_check" \
    --enable-negotiate-auth-helpers="mswin_sspi,squid_kerb_auth" \
    --enable-digest-auth-helpers="ldap,password" \
    --enable-external-acl-helpers="ip_user,ldap_group,mswin_lm_group,session,unix_group,wbinfo_group" \
    --enable-x-accelerator-vary \
    || exit -1

# compile package
make $ntasks DEFAULT_LOG_PREFIX=/var/log/squid DEFAULT_SWAP_DIR=/var/spool/squid \
DEFAULT_PID_FILE=/var/run/squid.pid

# install package to the target directory
make install DESTDIR=$dest

# copy the doc files

cp -a \
    CONTRIBUTORS COPYING COPYRIGHT CREDITS ChangeLog \
    INSTALL QUICKSTART README RELEASENOTES.html SPONSORS TODO \
    doc/debug-sections.txt \
    $dest/usr/doc/$softname-$softversion

echo "re-ordering ..."

# Add the default cache and logs folders
mkdir -p $dest/var/spool/squid
mkdir -p $dest/var/log/squid
# Remove the default log directory
rmdir $dest/var/logs


( cd $dest
    cp -a etc usr/share/$softname/
    ( cd etc/$softname
	for file in * ; do
#	    newfile=`echo ${file} | sed "s/\-example\.conf/\.conf\.new/g"`
	    newfile="${file}.new"
	    echo "${file} -> ${newfile}"
	    mv ${file} ${newfile}
	done
        # Make symlink for the default errors directory
	ln -s /usr/share/$softname/errors/English errors
    )
)


echo "creating rc script ..."
# put the initialization script in $dest/etc/rc.d/
mkdir -p $dest/etc/rc.d/desc.d
cat <<EOF  > $dest/etc/rc.d/rc.squid.new
#!/bin/bash
# squid         This shell script takes care of starting and stopping
#               Squid Internet Object Cache
#
# description: Squid - Internet Object Cache. Internet object caching is
#       a way to store requested Internet objects (i.e., data available
#       via the HTTP, FTP, and gopher protocols) on a system closer to the
#       requesting site than to the source. Web browsers can then use the
#       local Squid cache as a proxy HTTP server, reducing access time as
#       well as bandwidth consumption.
# pidfile: /var/run/squid.pid
# config: /etc/squid/squid.conf

PATH=/usr/bin:/sbin:/bin:/usr/sbin
export PATH

# don't raise an error if the config file is incomplete
# set defaults instead:
SQUID_OPTS=\${SQUID_OPTS:-"-D"}
SQUID_PIDFILE_TIMEOUT=\${SQUID_PIDFILE_TIMEOUT:-20}
SQUID_SHUTDOWN_TIMEOUT=\${SQUID_SHUTDOWN_TIMEOUT:-100}

# determine the name of the squid binary
[ -f /usr/sbin/squid ] && SQUID=squid
[ -z "\$SQUID" ] && exit 0

prog="\$SQUID"

# determine which one is the cache_swap directory
CACHE_SWAP=\`sed -e 's/#.*//g' /etc/squid/squid.conf | \\
        grep cache_dir |  awk '{ print \$3 }'\`
[ -z "\$CACHE_SWAP" ] && CACHE_SWAP=/var/spool/squid

RETVAL=0

start() {
    for adir in \$CACHE_SWAP; do
        if [ ! -d \$adir/00 ]; then
             echo -n "init_cache_dir $adir... "
             \$SQUID -z -F -D 2>/dev/null
        fi
    done
    echo
    echo -n "Starting $prog: "
    \$SQUID \$SQUID_OPTS 2> /dev/null
    RETVAL=\$?
    if [ \$RETVAL -eq 0 ]; then
       timeout=0;
       while : ; do
          [ ! -f /var/run/squid.pid ] || break
          if [ \$timeout -ge \$SQUID_PIDFILE_TIMEOUT ]; then
             RETVAL=1
             break
          fi
          sleep 1 && echo -n "."
          timeout=\$((timeout+1))
       done
    fi
    [ \$RETVAL -eq 0 ] && touch /var/lock/subsys/\$SQUID
    [ \$RETVAL -eq 0 ] && echo success
    [ \$RETVAL -ne 0 ] && echo failure
    echo
    return \$RETVAL
}

stop() {
    echo -n  "Stopping \$prog: "
    \$SQUID -k check >/dev/null 2>&1
    RETVAL=\$?
    if [ \$RETVAL -eq 0 ] ; then
        \$SQUID -k shutdown &
        rm -f /var/lock/subsys/\$SQUID
        timeout=0
        while : ; do
                [ -f /var/run/squid.pid ] || break
                if [ \$timeout -ge \$SQUID_SHUTDOWN_TIMEOUT ]; then
                    echo
                    return 1
                fi
                sleep 2 && echo -n "."
                timeout=\$((timeout+2))
        done
        echo success
        echo
    else
        echo failure
        echo
    fi
    return \$RETVAL
}

reload() {
    \$SQUID \$SQUID_OPTS -k reconfigure
}

restart() {
    stop
    start
}

condrestart() {
    [ -e /var/lock/subsys/squid ] && restart || :
}

status() {
    \$SQUID -k check
}

probe() {
    return 0
}

case "\$1" in
start)
    start
    ;;
stop)
    stop
    ;;
reload)
    reload
    ;;
restart)
    restart
    ;;
condrestart)
    condrestart
    ;;
status)
    status
    ;;
probe)
    exit 0
    ;;
*)
    echo "Usage: \$0 {start|stop|status|reload|restart|condrestart}"
    exit 1
esac
exit \$?
EOF


chmod 644 $dest/etc/rc.d/rc.$softname.new
chown root.root $dest/etc/rc.d/rc.$softname.new

mkdir -p $dest/etc/logrotate.d
cat <<EOF > $dest/etc/logrotate.d/$softname.new
/var/log/squid/access.log {
    weekly
    rotate 5
    copytruncate
    compress
    notifempty
    missingok
}
/var/log/squid/cache.log {
    weekly
    rotate 5
    copytruncate
    compress
    notifempty
    missingok
}

/var/log/squid/store.log {
    weekly
    rotate 5
    copytruncate
    compress
    notifempty
    missingok
# This script asks squid to rotate its logs on its own.
# Restarting squid is a long process and it is not worth
# doing it just to rotate logs
    postrotate
      /usr/sbin/squid -k rotate
    endscript
}
EOF

chmod 644 $dest/etc/logrotate.d/$softname.new
chown root.root $dest/etc/logrotate.d/$softname.new

# strip executables
echo "stripping ..."
( cd $dest
   find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
   find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)

##No man at all !!!
# gzip man pages and info files
echo "gzipping ..."
gzip -9 $dest/usr/man/*/*

# put the ZenWalk build and other needed files in the right directory
echo "setting perms ..."
cp -a $buildir/build-$softname.sh $dest/usr/src/$softname-$softversion
chown -R root.root $dest/usr/src/$softname-$softversion

# change files ownerships for $dest/usr/sbin
# NO $dest/usr/bin at all
chown -R root.root $dest/usr/bin
chown -R root.root $dest/usr/sbin

# make the package and move it to the build directory

# create doinst.sh file
echo "creating doinst.sh ..."
cat <<EOF >> $dest/install/doinst.sh
#! /bin/sh

config() {
  NEW="\$1"
  OLD="\`dirname \$NEW\`/\`basename \$NEW .new\`"
  # If there's no config file by that name, mv it over:
  if [ ! -r \$OLD ]; then
    mv \$NEW \$OLD
  elif [ "\`cat \$OLD | md5sum\`" = "\`cat \$NEW | md5sum\`" ]; then
    # toss the redundant copy
    rm \$NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}

config etc/rc.d/rc.squid.new

for file in etc/squid/* ;  do
  config \$file
done
config etc/logrotate.d/squid.new


# add a squid user and group
# Define minimum values for UID and GID
MUID=43
MGID=43
USERNAME=squid
GROUPNAME=squid
COMMENT=Squid
HOMEDIR=/var/spool/squid

(
if ! grep -qe "^\$GROUPNAME:" etc/group; then
        NEXTGID="\$(MGID=\$MGID awk -F: '{uid[\$3]=1} END { for (i=ENVIRON["MGID"];i in uid;i++);print i}' etc/group)"
        echo creating group \$GROUPNAME with GID \$NEXTGID...
        chroot . /usr/sbin/groupadd -g \$NEXTGID \$GROUPNAME
fi
)
(
if ! grep -qe "^\$USERNAME:" etc/passwd; then
        NEXTUID="\$(MUID=\$MUID awk -F: '{uid[\$3]=1} END { for (i=ENVIRON["MUID"];i in uid;i++);print i}' etc/passwd)"
        echo creating user \$USERNAME with UID \$NEXTUID...
        chroot . /usr/sbin/useradd -g \$GROUPNAME -u \$NEXTUID -d \$HOMEDIR -c \$COMMENT -s /bin/false \$USERNAME
fi
)

# fix rights and ownership on the squid cache directory
chown -R \$USERNAME:\$GROUPNAME var/spool/squid
chmod 0750 var/spool/squid

chown -R \$USERNAME:\$GROUPNAME var/log/squid
chmod 0750 var/log/squid

EOF

chmod 755 $dest/install/doinst.sh

# Create a desc file
echo "squid:The Squid Proxy Cache service:squid" > $dest/etc/rc.d/desc.d/squid.txt

# Create an out of the box working config file
sed -i '/#\ cache_effective_user\ nobody/a cache_effective_user squid
 ' $dest/etc/squid/squid.conf.new

#------------------( Create The *.dep File )
echo "creating .dep ..."
cd $dest
#requiredbuilder -z $dest > $buildir/$package.dep
echo -n "cxxlibs,cyrus-sasl,gcc,openldap-client,openssl,perl" > $buildir/$package.dep

cd $dest
/sbin/makepkg -l y -c n $package.tgz
mv $package.tgz $buildir

# computes the md5sum signature for the package
echo "creating .md5 ..."
cd $buildir
md5sum $package.tgz > $package.md5

echo "$(grep 'U[R]L' $0 | rev | cut -d ' ' -f1 | cut -d "	" -f 1 | rev)" > $package.src
echo "http://zen-repo.meticul.eu/zen_stuffs/guth/packages/$softname/build-$softname.sh" >> $package.src

if [ "$1" == "--cleanup" ]; then
  rm -rf $dest
  rm -rf $srcdir
fi

